@@ -25,7 +25,7 @@
 
 #include "config.h"
 
-//#define DODS_DEBUG 1
+// #define DODS_DEBUG
 
 #include <cassert>
 #include <sstream>
@@ -38,6 +38,8 @@
 #include "UInt16.h"
 #include "Int32.h"
 #include "UInt32.h"
+#include "Int64.h"
+#include "UInt64.h"
 
 #include "D4Group.h"
 #include "D4Enum.h"
@@ -76,6 +78,144 @@ void D4Enum::m_duplicate(const D4Enum &src)
     d_is_signed = src.d_is_signed;
 }
 
+
+/**
+ *
+ *
+ *
+ *
+ *
+ *
+ */
+std::vector<BaseType *> *
+D4Enum::transform_to_dap2(AttrTable *){
+    BaseType *my_pretty_pony;
+
+    DBG(cerr << __func__ << "() - BEGIN" << endl;);
+
+
+    switch (d_element_type) {
+    case dods_byte_c:
+    case dods_int8_c:
+    case dods_uint8_c:
+    {
+        Byte *var = new Byte(name());
+        dods_byte val;
+        this->value(&val);
+        var->set_value(val);
+        my_pretty_pony = var;
+        break;
+    }
+    case dods_uint16_c:
+    {
+        UInt16 *var = new UInt16(name());
+        dods_uint16 val;
+        this->value(&val);
+        var->set_value(val);
+        my_pretty_pony = var;
+        break;
+    }
+    case dods_uint32_c:
+    {
+        UInt32 *var = new UInt32(name());
+        dods_uint32 val;
+        this->value(&val);
+        var->set_value(val);
+        my_pretty_pony = var;
+        break;
+    }
+    case dods_uint64_c:
+    {
+        UInt64 *var = new UInt64(name());
+        dods_uint64 val;
+        this->value(&val);
+        var->set_value(val);
+        my_pretty_pony = var;
+        break;
+    }
+    case dods_int16_c:
+    {
+        Int16 *var = new Int16(name());
+        dods_int16 val;
+        this->value(&val);
+        var->set_value(val);
+        my_pretty_pony = var;
+        break;
+    }
+    case dods_int32_c:
+    {
+        Int32 *var = new Int32(name());
+        dods_int32 val;
+        this->value(&val);
+        var->set_value(val);
+        my_pretty_pony = var;
+        break;
+    }
+    case dods_int64_c:
+    {
+        Int64 *var = new Int64(name());
+        dods_int64 val;
+        this->value(&val);
+        var->set_value(val);
+        my_pretty_pony = var;
+        break;
+    }
+    default:
+    {
+        ostringstream oss;
+        oss << __func__ << "() - ERROR! Unknown D4Enum type:"<< d_element_type << " name: " <<  name() << endl;
+        throw InternalErr(__FILE__,__LINE__,oss.str());
+        break;
+    }
+    }
+
+    DBG( cerr << __func__ << "() - Processing Enum  type:"<<
+        my_pretty_pony->type_name() << " name: " << my_pretty_pony->name() << endl;);
+    /**
+     * Grab the attributes!
+     */
+    AttrTable d2_attrs = *(this->attributes()->get_AttrTable(name()));
+    my_pretty_pony->set_attr_table(d2_attrs);
+
+    // cerr << "D4Enum::transform_to_dap2() - my_pretty_pony attrs: "<< endl;
+    // my_pretty_pony->get_attr_table().print(cerr);
+
+    /**
+     * Now we make the Enum label, and the enum's definition
+     * into DAP2 attributes for our returned item.
+     */
+    long long my_value;
+    this->value(&my_value);
+    DBG(cerr << __func__ << "() - value: "<< my_value << endl;);
+
+    string my_label="";
+    AttrTable *enum_def = new AttrTable();
+    enum_def->set_name("d4:enum_def");
+
+    D4EnumDef::D4EnumValueIter dIter = d_enum_def->value_begin();
+    D4EnumDef::D4EnumValueIter dEnd = d_enum_def->value_end();
+    while( dIter!=dEnd){
+        long long a_value =  (*dIter).value;
+        string a_label =  (*dIter).label;
+        ostringstream oss;
+        oss << a_value;
+        DBG(cerr << __func__ << "() - a_value: "<< a_value << endl;);
+        enum_def->append_attr(a_label,my_pretty_pony->type_name(),oss.str());
+        if(a_value == my_value){
+            my_label = (*dIter).label;
+        }
+        dIter++;
+    }
+    if(!my_label.empty())
+        my_pretty_pony->get_attr_table().append_attr("d4:enum_label","String",my_label);
+    my_pretty_pony->get_attr_table().append_container(enum_def,enum_def->get_name());
+
+    vector<BaseType *> *result = new vector<BaseType *>();
+    result->push_back(my_pretty_pony);
+    DBG(cerr << __func__ << "() - END" << endl;);
+    return result;
+}
+
 void D4Enum::m_check_value(int64_t v) const
 {
 	switch (d_element_type) {
