@@ -74,7 +74,9 @@ public:
     virtual ~D4Group();
 
     D4Group &operator=(const D4Group &rhs);
-    virtual D4Group *ptr_duplicate();
+
+    // This method returned a D4Group * previously. jhrg 11/17/16
+    virtual BaseType *ptr_duplicate();
 
     /// Get the dimensions defined for this Group
     D4Dimensions *dims() {
@@ -137,6 +139,10 @@ public:
     virtual void deserialize(D4StreamUnMarshaller &um, DMR &dmr);
 
     void print_dap4(XMLWriter &xml, bool constrained = false);
+
+    virtual std::vector<BaseType *> *transform_to_dap2(AttrTable *parent_attr_table);
+    virtual std::vector<BaseType *> *transform_to_dap2(AttrTable *parent_attr_table, bool use_name_prefix);
+
 };
 
 } /* namespace libdap */
