@@ -22,6 +22,7 @@
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
 
+//#define DODS_DEBUG
 
 #include "config.h"
 
@@ -143,6 +144,13 @@ D4Opaque::value() const
     return d_buf;
 }
 
+std::vector<BaseType *> *
+D4Opaque::transform_to_dap2(AttrTable *){
+    DBG(cerr << __func__ << "() - Transform not implemented DAP4 Opaque type." << endl;);
+    return NULL;
+}
+
+
 void
 D4Opaque::print_val(ostream &out, string space, bool print_decl_p)
 {
