@@ -43,9 +43,9 @@
 #include "D4Attributes.h"
 #include "D4Maps.h"
 #include "D4Enum.h"
-
 #include "D4BaseTypeFactory.h"
 
+#include "DapXmlNamespaces.h"
 #include "D4ParserSax2.h"
 
 #include "util.h"
@@ -81,6 +81,8 @@ static const char *states[] = {
 
         "inside_constructor",
 
+        "not_dap4_element",
+
         "parser_unknown",
         "parser_error",
         "parser_fatal_error",
@@ -588,7 +590,7 @@ void D4ParserSax2::process_variable_helper(Type t, ParseState s, const xmlChar *
 void D4ParserSax2::dmr_start_document(void * p)
 {
     D4ParserSax2 *parser = static_cast<D4ParserSax2*>(p);
-    parser->error_msg = "";
+    parser->d_error_msg = "";
     parser->char_data = "";
 
     // Set this in intern_helper so that the loop test for the parser_end
@@ -624,6 +626,19 @@ void D4ParserSax2::dmr_end_document(void * p)
     parser->pop_attributes();
 }
 
+/**
+ * Callback run when libxml2 reads the start of an element
+ *
+ * @param p Pointer to the parser object
+ * @param l Localname of the element
+ * @param prefix Namespace prefix of the element
+ * @param URI the Element namespace name if available
+ * @param nb_namespaces Number of namespace definitions on that node
+ * @param namespaces Pointer to the array of prefix/URI pairs namespace definitions
+ * @param nb_attributes The number of attributes on that node
+ * @param nb_defaulted The number of defaulted attributes. The defaulted ones are at the end of the array
+ * @param attributes Pointer to the array of (localname/prefix/URI/value/end) attribute values.
+ */
 void D4ParserSax2::dmr_start_element(void *p, const xmlChar *l, const xmlChar *prefix, const xmlChar *URI,
         int nb_namespaces, const xmlChar **namespaces, int nb_attributes, int /*nb_defaulted*/,
         const xmlChar **attributes)
@@ -631,7 +646,23 @@ void D4ParserSax2::dmr_start_element(void *p, const xmlChar *l, const xmlChar *p
     D4ParserSax2 *parser = static_cast<D4ParserSax2*>(p);
     const char *localname = (const char *) l;
 
-    if (parser->debug()) cerr << "Start element " << localname << " (state " << states[parser->get_state()] << ")" << endl;
+    if (parser->debug()) cerr << "Start element " << localname << "  prefix:  "<< (prefix?(char *)prefix:"null") << "  ns: "<< (URI?(char *)URI:"null")
+    		   << " (state: " << states[parser->get_state()] << ")" << endl;
+
+    if(parser->get_state() != parser_error){
+        string dap4_ns_name = DapXmlNamspaces::getDapNamespaceString(DAP_4_0);
+        if (parser->debug()) cerr << "dap4_ns_name:         " << dap4_ns_name << endl;
+
+        string this_element_ns_name = (URI != 0) ? ((char *)URI) : "";
+        if (parser->debug()) cerr << "this_element_ns_name: " << this_element_ns_name << endl;
+
+        if(this_element_ns_name.compare(dap4_ns_name)){
+            if (parser->debug()) cerr << "Start of non DAP4 element: " << localname << " detected." << endl;
+        	parser->push_state(not_dap4_element);
+        	// return;
+        }
+    }
+
 
     switch (parser->get_state()) {
         case parser_start:
@@ -805,6 +836,10 @@ void D4ParserSax2::dmr_start_element(void *p, const xmlChar *l, const xmlChar *p
                 D4ParserSax2::dmr_error(parser, "Expected an Attribute, Dim, Map or variable element; found '%s' instead.", localname);
             break;
 
+        case not_dap4_element:
+            if (parser->debug()) cerr << "Inside non DAP4 element. localname: " << localname << endl;
+        	break;
+
         case parser_unknown:
             // FIXME?
             // *** Never used? If so remove/error
@@ -1053,6 +1088,11 @@ void D4ParserSax2::dmr_end_element(void *p, const xmlChar *l, const xmlChar *pre
         break;
     }
 
+    case not_dap4_element:
+        if (parser->debug()) cerr << "End of non DAP4 element: " << localname << endl;
+        parser->pop_state();
+    	break;
+
     case parser_unknown:
         parser->pop_state();
         break;
@@ -1165,10 +1205,10 @@ void D4ParserSax2::dmr_fatal_error(void * p, const char *msg, ...)
     vsnprintf(str, 1024, msg, args);
     va_end(args);
 
-    int line = xmlSAX2GetLineNumber(parser->context);
+    int line = xmlSAX2GetLineNumber(parser->d_context);
 
-    if (!parser->error_msg.empty()) parser->error_msg += "\n";
-    parser->error_msg += "At line " + long_to_string(line) + ": " + string(str);
+    if (!parser->d_error_msg.empty()) parser->d_error_msg += "\n";
+    parser->d_error_msg += "At line " + long_to_string(line) + ": " + string(str);
 }
 
 void D4ParserSax2::dmr_error(void *p, const char *msg, ...)
@@ -1183,10 +1223,10 @@ void D4ParserSax2::dmr_error(void *p, const char *msg, ...)
     vsnprintf(str, 1024, msg, args);
     va_end(args);
 
-    int line = xmlSAX2GetLineNumber(parser->context);
+    int line = xmlSAX2GetLineNumber(parser->d_context);
 
-    if (!parser->error_msg.empty()) parser->error_msg += "\n";
-    parser->error_msg += "At line " + long_to_string(line) + ": " + string(str);
+    if (!parser->d_error_msg.empty()) parser->d_error_msg += "\n";
+    parser->d_error_msg += "At line " + long_to_string(line) + ": " + string(str);
 }
 //@}
 
@@ -1195,11 +1235,11 @@ void D4ParserSax2::dmr_error(void *p, const char *msg, ...)
  */
 void D4ParserSax2::cleanup_parse()
 {
-    bool wellFormed = context->wellFormed;
-    bool valid = context->valid;
+    bool wellFormed = d_context->wellFormed;
+    bool valid = d_context->valid;
 
-    context->sax = NULL;
-    xmlFreeParserCtxt(context);
+    d_context->sax = NULL;
+    xmlFreeParserCtxt(d_context);
 
     delete d_enum_def;
     d_enum_def = 0;
@@ -1215,13 +1255,13 @@ void D4ParserSax2::cleanup_parse()
     }
 
     if (!wellFormed)
-        throw Error("The DMR was not well formed. " + error_msg);
+        throw Error("The DMR was not well formed. " + d_error_msg);
     else if (!valid)
-        throw Error("The DMR was not valid." + error_msg);
+        throw Error("The DMR was not valid." + d_error_msg);
     else if (get_state() == parser_error)
-        throw Error(error_msg);
+        throw Error(d_error_msg);
     else if (get_state() == parser_fatal_error)
-        throw InternalErr(error_msg);
+        throw InternalErr(d_error_msg);
 }
 
 /**
@@ -1251,6 +1291,8 @@ void D4ParserSax2::intern(istream &f, DMR *dest_dmr, bool debug)
 
     d_dmr = dest_dmr; // dump values here
 
+#if 0
+    // Old, bad, code. Lines are limited to 1023 chars including the element text.
     const int size = 1024;
     char chars[size];
     int line = 1;
@@ -1259,21 +1301,56 @@ void D4ParserSax2::intern(istream &f, DMR *dest_dmr, bool debug)
     int res = f.gcount();
     if (res == 0) throw Error("No input found while parsing the DMR.");
 
+    getline(f, line);
+
     if (debug) cerr << "line: (" << line++ << "): " << chars << endl;
 
-    context = xmlCreatePushParserCtxt(&ddx_sax_parser, this, chars, res - 1, "stream");
-    context->validate = true;
+    d_context = xmlCreatePushParserCtxt(&d_dmr_sax_parser, this, chars, res - 1, "stream");
+    d_context->validate = true;
     push_state(parser_start);
 
     f.getline(chars, size);
     while ((f.gcount() > 0) && (get_state() != parser_end)) {
         if (debug) cerr << "line: (" << line++ << "): " << chars << endl;
-        xmlParseChunk(context, chars, f.gcount() - 1, 0);
+        xmlParseChunk(d_context, chars, f.gcount() - 1, 0);
         f.getline(chars, size);
     }
 
     // This call ends the parse.
-    xmlParseChunk(context, chars, 0, 1/*terminate*/);
+    xmlParseChunk(d_context, chars, 0, 1/*terminate*/);
+#else
+    int line_num = 1;
+    string line;
+
+    // Get the <xml ... ?> line
+    getline(f, line);
+    if (line.length() == 0) throw Error("No input found while parsing the DMR.");
+
+    if (debug) cerr << "line: (" << line_num << "): " << endl << line << endl << endl;
+
+    d_context = xmlCreatePushParserCtxt(&d_dmr_sax_parser, this, line.c_str(), line.length(), "stream");
+    d_context->validate = true;
+    push_state(parser_start);
+
+    // Get the first line of stuff
+    getline(f, line);
+    ++line_num;
+
+    if (debug) cerr << "line: (" << line_num << "): " << endl << line << endl << endl;
+
+    while (!f.eof() && (get_state() != parser_end)) {
+        xmlParseChunk(d_context, line.c_str(), line.length(), 0);
+
+        // Get the next line
+        getline(f, line);
+        ++line_num;
+
+        if (debug) cerr << "line: (" << line_num << "): " << endl << line << endl << endl;
+    }
+
+    // This call ends the parse.
+    xmlParseChunk(d_context, line.c_str(), 0, 1/*terminate*/);
+#endif
 
     // This checks that the state on the parser stack is parser_end and throws
     // an exception if it's not (i.e., the loop exited with gcount() == 0).
@@ -1317,13 +1394,11 @@ void D4ParserSax2::intern(const char *buffer, int size, DMR *dest_dmr, bool debu
     d_dmr = dest_dmr; // dump values in dest_dmr
 
     push_state(parser_start);
-    context = xmlCreatePushParserCtxt(&ddx_sax_parser, this, buffer, size, "stream");
-    context->validate = true;
-    //push_state(parser_start);
-    //xmlParseChunk(context, buffer, size, 0);
+    d_context = xmlCreatePushParserCtxt(&d_dmr_sax_parser, this, buffer, size, "stream");
+    d_context->validate = true;
 
     // This call ends the parse.
-    xmlParseChunk(context, buffer, 0, 1/*terminate*/);
+    xmlParseChunk(d_context, buffer, 0, 1/*terminate*/);
 
     // This checks that the state on the parser stack is parser_end and throws
     // an exception if it's not (i.e., the loop exited with gcount() == 0).
