@@ -83,6 +83,8 @@ public:
     virtual void print_val(ostream &out, string space = "", bool print_decl_p = true);
 
     virtual bool ops(BaseType *b, int op);
+    virtual bool d4_ops(BaseType *b, int op);
+    virtual std::vector<BaseType *> *transform_to_dap2(AttrTable *parent_attr_table);
 
     virtual void dump(ostream &strm) const ;
 };
