@@ -5,8 +5,9 @@
 // This file is part of libdap, A C++ implementation of the OPeNDAP Data
 // Access Protocol.
 
-// Copyright (c) 2002,2003 OPeNDAP, Inc.
+// Copyright (c) 2002,2003,2016 OPeNDAP, Inc.
 // Author: Patrick West <pwest@ucar.edu>
+//         James Gallagher <jgallagher@opendap.org>
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -30,6 +31,7 @@
 // Authors:
 //      pwest       Patrick West <pwest@ucar.edu>
 
+
 #include "config.h"
 
 #ifdef HAVE_PTHREAD_H
@@ -63,7 +65,7 @@ using namespace std;
 namespace libdap {
 
 char *XDRStreamMarshaller::d_buf = 0;
-#define XDR_DAP_BUFF_SIZE 256
+static const int XDR_DAP_BUFF_SIZE=256;
 
 
 /** Build an instance of XDRStreamMarshaller. Bind the C++ stream out to this
@@ -78,7 +80,7 @@ XDRStreamMarshaller::XDRStreamMarshaller(ostream &out) :
     d_out(out), d_partial_put_byte_count(0), tm(0)
 {
     if (!d_buf) d_buf = (char *) malloc(XDR_DAP_BUFF_SIZE);
-    if (!d_buf) throw Error("Failed to allocate memory for data serialization.");
+    if (!d_buf) throw Error(internal_error, "Failed to allocate memory for data serialization.");
 
     xdrmem_create(&d_sink, d_buf, XDR_DAP_BUFF_SIZE, XDR_ENCODE);
 
@@ -89,16 +91,18 @@ XDRStreamMarshaller::XDRStreamMarshaller(ostream &out) :
 
 XDRStreamMarshaller::~XDRStreamMarshaller()
 {
+    // Added this because when USE_POS... is not defined, 'tm' has no
+    // type, which the compiler complains about.
+#ifdef USE_POSIX_THREADS
     delete tm;
-
+#endif
     xdr_destroy(&d_sink);
 }
 
 void XDRStreamMarshaller::put_byte(dods_byte val)
 {
      if (!xdr_setpos(&d_sink, 0))
-        throw Error(
-            "Network I/O Error. Could not send byte data - unable to set stream position.");
+        throw Error("Network I/O Error. Could not send byte data - unable to set stream position.");
 
     if (!xdr_char(&d_sink, (char *) &val))
         throw Error(
@@ -212,7 +216,7 @@ void XDRStreamMarshaller::put_uint16(dods_uint16 val)
 
     if (!XDR_UINT16(&d_sink, &val))
         throw Error(
-            "Network I/O Error. Could not send uint 16 data. This may be due to a\nbug in libdap or a problem with the network connection.");
+            "Network I/O Error. Could not send uint 16 data.");
 
     unsigned int bytes_written = xdr_getpos(&d_sink);
     if (!bytes_written)
@@ -234,7 +238,7 @@ void XDRStreamMarshaller::put_uint32(dods_uint32 val)
 
     if (!XDR_UINT32(&d_sink, &val))
         throw Error(
-            "Network I/O Error. Could not send uint 32 data. This may be due to a\nbug in libdap or a problem with the network connection.");
+            "Network I/O Error. Could not send uint 32 data.");
 
     unsigned int bytes_written = xdr_getpos(&d_sink);
     if (!bytes_written)
@@ -420,14 +424,14 @@ void XDRStreamMarshaller::put_vector(char *val, int num, Vector &)
 #else
         d_out.write(byte_buf, bytes_written);
         xdr_destroy(&byte_sink);
-        delete byte_buf;
+        delete [] byte_buf;
 #endif
 
     }
     catch (...) {
         DBG(cerr << "Caught an exception in put_vector_thread" << endl);
         xdr_destroy(&byte_sink);
-        delete byte_buf;
+        delete [] byte_buf;
         throw;
     }
 }
@@ -445,11 +449,14 @@ void XDRStreamMarshaller::put_vector(char *val, int num, Vector &)
  */
 void XDRStreamMarshaller::put_vector(char *val, unsigned int num, int width, Type type)
 {
-    if (!val) throw InternalErr(__FILE__, __LINE__, "Buffer pointer is not set.");
+    assert(val || num == 0);
 
     // write the number of array members being written, then set the position back to 0
     put_int(num);
 
+    if (num == 0)
+        return;
+
     int use_width = width;
     if (use_width < 4) use_width = 4;
 
@@ -485,12 +492,12 @@ void XDRStreamMarshaller::put_vector(char *val, unsigned int num, int width, Typ
 #else
         d_out.write(vec_buf, bytes_written);
         xdr_destroy(&vec_sink);
-        delete vec_buf;
+        delete [] vec_buf;
 #endif
     }
     catch (...) {
         xdr_destroy(&vec_sink);
-        delete vec_buf;
+        delete [] vec_buf;
         throw;
     }
 }
@@ -546,12 +553,12 @@ void XDRStreamMarshaller::put_vector_part(char *val, unsigned int num, int width
             d_partial_put_byte_count += num;
 
             xdr_destroy(&byte_sink);
-            delete byte_buf;
+            delete [] byte_buf;
 #endif
         }
         catch (...) {
             xdr_destroy(&byte_sink);
-            delete byte_buf;
+            delete [] byte_buf;
             throw;
         }
     }
@@ -597,12 +604,12 @@ void XDRStreamMarshaller::put_vector_part(char *val, unsigned int num, int width
             d_partial_put_byte_count += (size - 4);
 
             xdr_destroy(&vec_sink);
-            delete vec_buf;
+            delete [] vec_buf;
 #endif
         }
         catch (...) {
             xdr_destroy(&vec_sink);
-            delete vec_buf;
+            delete [] vec_buf;
             throw;
         }
     }
