@@ -133,11 +133,17 @@ unsigned int TestArray::m_print_array(ostream &out, unsigned int index, unsigned
 {
     if (dims == 1) {
         out << "{";
-        for (unsigned i = 0; i < shape[0] - 1; ++i) {
+
+        // Added this test for zero-length arrays. jhrg 1/28/16
+        if (shape[0] >= 1) {
+            for (unsigned i = 0; i < shape[0] - 1; ++i) {
+                dynamic_cast<TestCommon&>(*var(index++)).output_values(out);
+                out << ", ";
+            }
+
             dynamic_cast<TestCommon&>(*var(index++)).output_values(out);
-            out << ", ";
         }
-        dynamic_cast<TestCommon&>(*var(index++)).output_values(out);
+
         out << "}";
 
         return index;
@@ -148,13 +154,21 @@ unsigned int TestArray::m_print_array(ostream &out, unsigned int index, unsigned
         // length is shape[dims-1]-1 *and* since we want one less dimension
         // than that, the correct limit on this loop is shape[dims-2]-1. From
         // Todd Karakasian.
+        //
         // The saga continues; the loop test should be `i < shape[0]-1'. jhrg
         // 9/12/96.
-        for (unsigned i = 0; i < shape[0] - 1; ++i) {
+        //
+        // Added this (primitive) guard against errors when a zero-length array
+        // is declared with a shape like [0][4]. jhrg 1/28/16
+        if (shape[0] > 0) {
+            for (unsigned i = 0; i < shape[0] - 1; ++i) {
+                index = m_print_array(out, index, dims - 1, shape + 1);
+                out << ",";
+            }
+
             index = m_print_array(out, index, dims - 1, shape + 1);
-            out << ",";
         }
-        index = m_print_array(out, index, dims - 1, shape + 1);
+
         out << "}";
 
         return index;
@@ -411,44 +425,54 @@ bool TestArray::read()
     //case dods_uint8_c:
     case dods_int16_c:
         m_cardinal_type_read_helper<dods_int16, Int16>();
+        set_read_p(true);
         break;
 
     case dods_uint16_c:
         m_cardinal_type_read_helper<dods_uint16, UInt16>();
+        set_read_p(true);
         break;
 
     case dods_int32_c:
         m_cardinal_type_read_helper<dods_int32, Int32>();
+        set_read_p(true);
         break;
 
     case dods_uint32_c:
         m_cardinal_type_read_helper<dods_uint32, UInt32>();
+        set_read_p(true);
         break;
 
     case dods_float32_c:
         m_cardinal_type_read_helper<dods_float32, Float32>();
+        set_read_p(true);
         break;
 
     case dods_float64_c:
         m_cardinal_type_read_helper<dods_float64, Float64>();
+        set_read_p(true);
         break;
 
     case dods_int8_c:
         m_cardinal_type_read_helper<dods_int8, Int8>();
+        set_read_p(true);
         break;
 
     case dods_byte_c:
     case dods_char_c:
     case dods_uint8_c:
         m_cardinal_type_read_helper<dods_byte, Byte>();
+        set_read_p(true);
         break;
 
     case dods_int64_c:
         m_cardinal_type_read_helper<dods_int64, Int64>();
+        set_read_p(true);
         break;
 
     case dods_uint64_c:
         m_cardinal_type_read_helper<dods_uint64, UInt64>();
+        set_read_p(true);
         break;
 
     case dods_enum_c:
@@ -482,6 +506,7 @@ bool TestArray::read()
         default:
             throw InternalErr(__FILE__, __LINE__, "Enum with undefined type.");
         }
+        set_read_p(true);
         break;
 
     case dods_str_c:
@@ -505,31 +530,31 @@ bool TestArray::read()
         }
 
         set_value(tmp, array_len);
+        set_read_p(true);
         break;
     }
 
     case dods_opaque_c:
     case dods_structure_c:
+        vec_resize(array_len);
         for (unsigned i = 0; i < array_len; ++i) {
             // Copy the prototype and read a value into it
             BaseType *elem = var()->ptr_duplicate();
             elem->read();
             // Load the new value into this object's array
-            set_vec(i, elem);
+            set_vec_nocopy(i, elem);   // Use set_vec_nocopy() TODO (and below)
         }
-
+        set_read_p(true);
         break;
 
     case dods_sequence_c:
         // No sequence arrays in DAP2
         if (!is_dap4()) throw InternalErr(__FILE__, __LINE__, "Bad data type");
 
+        vec_resize(array_len);
         for (unsigned i = 0; i < array_len; ++i) {
-            // Copy the prototype and read a value into it
-            BaseType *elem = var()->ptr_duplicate();
-            //elem->read();
-            // Load the new value into this object's array
-            set_vec(i, elem);
+            // Load the new BaseType (a D4Sequence) into the array element
+            set_vec_nocopy(i, var()->ptr_duplicate());
         }
 
         break;
@@ -543,7 +568,7 @@ bool TestArray::read()
         break;
     }
 
-    set_read_p(true);
+    // set_read_p(true);
 
     return true;
 }
